@@ -25,7 +25,7 @@ def _parse_vaccinations_timestamp(data):
         # ? raise exception or return error obj
         return error
     resp = data["results"][0]["result"]["data"]["dsr"]["DS"][0]["PH"][0]["DM0"]
-    return resp[0]["M0"]
+    return parse_date(resp[0]["M0"])
 
 
 def _parse_vaccinations_by_day(data) -> "list[VaccinationByDayRow]":
@@ -274,7 +274,7 @@ def _parse_vaccines_supplied_by_manufacturer_cum(
     return parsed_data
 
 
-def _parse_vaccinations_by_age_group(data) -> "list[vaccinations_by_age_group]":
+def _parse_vaccinations_by_age_group(data) -> "list[VaccinationByDayRow]":
     resp = data["results"][0]["result"]["data"]["dsr"]["DS"][0]["PH"][0]["DM0"]
     parsed_data = []
 
@@ -371,38 +371,35 @@ def _parse_vaccinations_by_municipalities_share(data) -> "list[VaccinationMunSha
         R = el.get("R", None)
         if len(C) == 6:
             name, population, share2, share1, dose1, dose2 = el["C"]
-            parsed_data.append(
-                VaccinationMunShare(
-                    name=name,
-                    dose1=dose1,
-                    share1=float(share1),
-                    dose2=dose2,
-                    share2=float(share2),
-                    population=population,
-                )
-            )
+
         elif len(C) == 5:
             if R == 32:
                 name, population, share2, share1, dose1 = el["C"]
                 dose2 = int(population * float(share2))
                 print(dose2)
-                parsed_data.append(
-                    VaccinationMunShare(
-                        name=name,
-                        dose1=dose1,
-                        share1=float(share1),
-                        dose2=dose2,
-                        share2=float(share2),
-                        population=population,
-                    )
-                )
+            elif R == 16:
+                name, population, share2, share1, dose2 = el["C"]
+                dose1 = int(population * float(share1))
+                print(dose1)
             else:
                 print(el)
                 raise Exception(f"Unknown R: {R}")
+
         else:
             print(el)
             raise Exception(f"Unknown C length: {len(C)}")
 
+        parsed_data.append(
+            VaccinationMunShare(
+                name=name,
+                dose1=dose1,
+                share1=float(share1),
+                dose2=dose2,
+                share2=float(share2),
+                population=population,
+            )
+        )
+
     return parsed_data
 
 
@@ -499,4 +496,125 @@ def _parse_vaccinations_gender_by_date(data):
         raise Exception("Something went wrong!")
 
     resp = data["results"][0]["result"]["data"]["dsr"]["DS"][0]["PH"][0]["DM0"]
-    return resp[0].get("M0", None)
\ No newline at end of file
+    return resp[0].get("M0", None)
+
+
+# date range parsers
+def _parse_vaccinations_date_range(data):
+    if "DS" not in data["results"][0]["result"]["data"]["dsr"]:
+        error = data["results"][0]["result"]["data"]["dsr"]["DataShapes"][0][
+            "odata.error"
+        ]
+        print(error)
+        raise Exception("Something went wrong!")
+
+    resp = data["results"][0]["result"]["data"]["dsr"]["DS"][0]["PH"][0]["DM0"]
+
+    parsed_data = []
+
+    date = None
+    people_vaccinated = None
+    people_fully_vaccinated = None
+    for element in resp:
+        C = element["C"]
+        if len(C) == 3:
+            date = parse_date(C[0])
+            people_vaccinated = C[1]
+            people_fully_vaccinated = C[2]
+        elif len(C) == 2:
+            date = parse_date(C[0])
+            R = element["R"]
+            if R == 2:
+                people_fully_vaccinated = C[1]
+            else:
+                people_vaccinated = C[1]
+        elif len(C) == 1:
+            # R == 6
+            date = parse_date(C[0])
+        else:
+            raise Exception("Unknown item length!")
+
+        parsed_data.append(
+            VaccinationByDayRow(
+                date=date,
+                first_dose=people_vaccinated,
+                second_dose=people_fully_vaccinated,
+            )
+        )
+
+    return parsed_data
+
+
+def _parse_vaccinations_by_manufacturer_used(data):
+    if "DS" not in data["results"][0]["result"]["data"]["dsr"]:
+        error = data["results"][0]["result"]["data"]["dsr"]["DataShapes"][0][
+            "odata.error"
+        ]
+        print(error)
+        raise Exception("Something went wrong!")
+
+    resp = data["results"][0]["result"]["data"]["dsr"]["DS"][0]["PH"][0]["DM0"]
+
+    manu_dict = {0: "az", 1: "janssen", 2: "moderna", 3: "pfizer"}
+
+    parsed_data = []
+    day_delta = datetime.timedelta(days=1)
+    previous_date = datetime.datetime(2020, 12, 26)
+    for index, element in enumerate(resp):
+        date = parse_date(element["G0"])
+        possible_missing_date = previous_date + day_delta
+        while possible_missing_date < date:
+            print(f"Add data for missing date: {possible_missing_date}")
+            parsed_data.append(
+                VaccinationByManufacturerRow(
+                    possible_missing_date, None, None, None, None
+                )
+            )
+            possible_missing_date += day_delta
+        previous_date = date
+
+        obj = VaccinationByManufacturerRow(date, None, None, None, None)
+        X = [el for el in element["X"] if el.get("M0", None) != None]
+        if len(X) == 4:
+            obj.az = X[0]["M0"]
+            obj.janssen = X[1]["M0"]
+            obj.moderna = X[2]["M0"]
+            obj.pfizer = X[3]["M0"]
+        else:
+            onlyTwoManufacturersDate = datetime.datetime(2021, 2, 3)
+            if date < onlyTwoManufacturersDate:
+                if len(X) == 1:
+                    I = X[0].get("I", None)
+                    M0 = X[0].get("M0", None)
+                    if I == None:
+                        obj.moderna = M0
+                    else:
+                        key = manu_dict[I]
+                        obj.__setattr__(key, M0)
+                elif len(X) == 2:
+                    obj.moderna = X[0]["M0"]
+                    obj.pfizer = X[1]["M0"]
+                else:
+                    print(X)
+                    raise Exception(
+                        "This is strange! There should be data only for 2 manufacturers!"
+                    )
+            else:
+                lastI = None
+                for index, item in enumerate(X):
+                    I = item.get("I", None)
+                    M0 = item.get("M0", None)
+
+                    if I != None:
+                        key = manu_dict[I]
+                        obj.__setattr__(key, M0)
+                    elif lastI == None:
+                        key = manu_dict[index]
+                        obj.__setattr__(key, M0)
+                    else:
+                        key = manu_dict[lastI + 1]
+                        obj.__setattr__(key, M0)
+                    lastI = I
+        parsed_data.append(obj)
+
+    return parsed_data
